"""\
Error report service

Sometimes an Exception occurs rendering with the ``cgitb`` module.
"""

import logging
import cgitb
import sys
import traceback
import StringIO

from bn import AttributeDict, str_dict

log = logging.getLogger(__name__)

class ErrorReportService(object):
    def __init__(
        self,
        to, 
        from_email='error@example.com', 
        from_name='ErrorReportMiddleware',
        subject='Error Report',
    ):
        self.to = to
        self.from_email = from_email
        self.from_name = from_name
        self.subject = subject

    @staticmethod
    def config(flow, name):
        from configconvert import handle_option_error, handle_section_error
        if not flow.config.option_group.has_key(name):
            raise handle_section_error(
                flow, 
                name, 
                "'%s.to', '%s.from_email' and '%s.subject'"%(
                    name, 
                    name, 
                    name
                )
            )
        from conversionkit import chainConverters, Conversion
        from stringconvert import unicodeToUnicode
        from stringconvert.email import listOfEmails, unicodeToEmail
        from recordconvert import toRecord

        to_unicode = unicodeToUnicode()
        # Error Reporting
        error_mail_converters={
            'subject': to_unicode,
            'to': listOfEmails(),
            'from_email': unicodeToEmail(),
            'from_name': to_unicode,
        }
        error_mail_converter = chainConverters(
            toRecord(
                missing_errors = {
                    'to': "The required option '%s.to' is missing"%name,
                },
                empty_errors = {
                    'to': "The option '%s.to' cannot be empty"%name,
                },
                converters = error_mail_converters,
            ),
        )
        conversion = Conversion(flow.config.option_group[name]).perform(
            error_mail_converter
        )
        if not conversion.successful:
            handle_option_error(conversion, name)
        else:
            flow.config[name] = conversion.result
        return flow.config[name]

    @staticmethod
    def create(flow, name, config=None):
        if config is None:
            config = ErrorReportService.config(flow, name)
        return ErrorReportService(**str_dict(config))

    def start(self, flow, key):

        def report(exc_info=None):
            if not hasattr(flow, 'mail'):
                flow.provider.start('mail')
            # Use the mail service to send a report
            flow.mail.send(
                display_html(exc_info),
                to=self.to,
                from_email=self.from_email,
                from_name=self.from_name,
                subject=self.subject,
                type='html',
            )

        def display_html(exc_info=None):
            if exc_info is None:
                exc_info = sys.exc_info()
            # Get the traceback information
            return cgitb.html(exc_info)

        flow[key] = AttributeDict()
        flow[key]['report'] = report
        flow[key]['display_html'] = display_html

    def error(self, flow, key):
        original_error = sys.exc_info()
        msg = (
            'Could not show an error report because the HTTP service is not '
            'present'
        )
        try:
            if not flow.config.app.debug:
                if not hasattr(flow, 'http'):
                    log.error(msg)
                else:
                    if not hasattr(flow, 'error_document'):
                        flow.provider.start('error_document')
                    flow.error_document.render(status='500 An error occurred')
                    flow.error_report.report()
            else:
                if not hasattr(flow, 'http'):
                    log.error(msg)
                else:
                    flow.http.response.status = '500 Internal Server Error'
                    flow.http.response.headers = [
                        ('Content-type', 'text/html; charset=utf-8')
                    ]
                    flow.http.response.body = [flow[key].display_html(original_error)]
            flow.provider['error_handled'] = True
        except Exception, e:
            flow.http.response.headers = [
                ('Content-type', 'text/plain; charset=utf-8')
            ]
            try:
                fp = StringIO.StringIO()
                traceback.print_tb(original_error[2], file=fp)
                tb = fp.getvalue()
                fp.close()
            except:
                flow.http.response.status = \
                   '500 ErrorReportService Internal Server Error'
                flow.http.response.body = ['Error']
            else:
                log.error(
                    'Exception happened handling error: %r. Original: %r\n%r'%(
                        e, 
                        original_error[1],
                        tb, 
                    )
                )
                if flow.config.app.debug:
                    flow.http.response.body = [tb]
                else:
                    flow.http.response.body = ['Error']
                flow.provider['error_handled'] = True

